from __future__ import annotations

from typing import NewType

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

H5Element = NewType("H5Element", Element)
"""A `<h5>` element."""


def h5(
    *children: Child,
    extra_attrs: Attributes | None = None,
    **attrs: Unpack[GlobalAttrs],
) -> H5Element:
    """Represents fifth level section heading.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/Heading_Elements>
    """
    el = Element("h5")(**attrs, **extra_attrs or {})(*children)
    return H5Element(el)



from __future__ import annotations

from typing import NewType

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

SampElement = NewType("SampElement", Element)
"""A `<samp>` element."""


def samp(
    *children: Child,
    extra_attrs: Attributes | None = None,
    **attrs: Unpack[GlobalAttrs],
) -> SampElement:
    """Encloses inline text representing sample output from a computer program.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/samp>
    """
    el = Element("samp")(**attrs, **extra_attrs or {})(*children)
    return SampElement(el)



from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import prettify


def test_table(snapshot: Snapshot) -> None:
    dom = H.table(
        H.caption("Employee information"),
        H.colgroup(
            H.col(span=2),
        ),
        H.thead(
            H.tr(
                H.th("Name"),
                H.th("Age"),
            )
        ),
        H.tbody(
            H.tr(
                H.td("Maria Sanchez"),
                H.td("28"),
            ),
            H.tr(
                H.td("Michael Johnson"),
                H.td("34"),
            ),
        ),
        H.tfoot(
            H.tr(
                H.th("Average"),
                H.td("31"),
            )
        ),
    )

    snapshot.assert_match(prettify(dom), "test_table.html")



from __future__ import annotations

from typing import NewType

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

TbodyElement = NewType("TbodyElement", Element)
"""A `<tbody>` element."""


def tbody(
    *children: Child,
    extra_attrs: Attributes | None = None,
    **attrs: Unpack[GlobalAttrs],
) -> TbodyElement:
    """Indicates that table rows comprise the table's (main) data.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tbody>
    """
    el = Element("tbody")(**attrs, **extra_attrs or {})(*children)
    return TbodyElement(el)



from typing import Union

from bs4 import BeautifulSoup as bs
from bs4.formatter import HTMLFormatter

from haitch import SupportsHtml


def prettify(dom: SupportsHtml) -> Union[str, bytes]:
    """Pretty format rendered HTML to make it easier to spot diffs."""
    formatter = HTMLFormatter(indent=2, empty_attributes_are_booleans=True)
    soup = bs(str(dom), features="html.parser")
    return soup.prettify(formatter=formatter)
